/**
 * @license
 * Copyright 2017 The Lighthouse Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _util = _interopRequireDefault(require("./util.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e2) { throw _e2; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e3) { didErr = true; err = _e3; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/* globals self, Util */

/** @typedef {import('./dom.js')} DOM */

/** @typedef {import('./report-renderer.js')} ReportRenderer */

/** @typedef {import('./details-renderer.js')} DetailsRenderer */

/** @typedef {import('./util.js')} Util */

/** @typedef {'failed'|'warning'|'manual'|'passed'|'notApplicable'} TopLevelClumpId */
var CategoryRenderer = /*#__PURE__*/function () {
  /**
   * @param {DOM} dom
   * @param {DetailsRenderer} detailsRenderer
   */
  function CategoryRenderer(dom, detailsRenderer) {
    _classCallCheck(this, CategoryRenderer);

    /** @type {DOM} */
    this.dom = dom;
    /** @type {DetailsRenderer} */

    this.detailsRenderer = detailsRenderer;
    /** @type {ParentNode} */

    this.templateContext = this.dom.document();
    this.detailsRenderer.setTemplateContext(this.templateContext);
  }
  /**
   * Display info per top-level clump. Define on class to avoid race with Util init.
   */


  _createClass(CategoryRenderer, [{
    key: "renderAudit",

    /**
     * @param {LH.ReportResult.AuditRef} audit
     * @return {Element}
     */
    value: function renderAudit(audit) {
      var tmpl = this.dom.cloneTemplate('#tmpl-lh-audit', this.templateContext);
      return this.populateAuditValues(audit, tmpl);
    }
    /**
     * Populate an DOM tree with audit details. Used by renderAudit and renderOpportunity
     * @param {LH.ReportResult.AuditRef} audit
     * @param {DocumentFragment} tmpl
     * @return {!Element}
     */

  }, {
    key: "populateAuditValues",
    value: function populateAuditValues(audit, tmpl) {
      var _this = this;

      var strings = _util.default.i18n.strings;
      var auditEl = this.dom.find('.lh-audit', tmpl);
      auditEl.id = audit.result.id;
      var scoreDisplayMode = audit.result.scoreDisplayMode;

      if (audit.result.displayValue) {
        this.dom.find('.lh-audit__display-text', auditEl).textContent = audit.result.displayValue;
      }

      var titleEl = this.dom.find('.lh-audit__title', auditEl);
      titleEl.appendChild(this.dom.convertMarkdownCodeSnippets(audit.result.title));
      this.dom.find('.lh-audit__description', auditEl).appendChild(this.dom.convertMarkdownLinkSnippets(audit.result.description));

      if (audit.stackPacks) {
        audit.stackPacks.forEach(function (pack) {
          var packElm = _this.dom.createElement('div');

          packElm.classList.add('lh-audit__stackpack');

          var packElmImg = _this.dom.createElement('img');

          packElmImg.classList.add('lh-audit__stackpack__img');
          packElmImg.src = pack.iconDataURL;
          packElmImg.alt = pack.title;
          packElm.appendChild(packElmImg);
          packElm.appendChild(_this.dom.convertMarkdownLinkSnippets(pack.description));

          _this.dom.find('.lh-audit__stackpacks', auditEl).appendChild(packElm);
        });
      }

      var header =
      /** @type {HTMLDetailsElement} */
      this.dom.find('details', auditEl);

      if (audit.result.details) {
        var elem = this.detailsRenderer.render(audit.result.details);

        if (elem) {
          elem.classList.add('lh-details');
          header.appendChild(elem);
        }
      } // Add chevron SVG to the end of the summary


      this.dom.find('.lh-chevron-container', auditEl).appendChild(this._createChevron());

      this._setRatingClass(auditEl, audit.result.score, scoreDisplayMode);

      if (audit.result.scoreDisplayMode === 'error') {
        auditEl.classList.add("lh-audit--error");
        var textEl = this.dom.find('.lh-audit__display-text', auditEl);
        textEl.textContent = strings.errorLabel;
        textEl.classList.add('tooltip-boundary');
        var tooltip = this.dom.createChildOf(textEl, 'div', 'tooltip tooltip--error');
        tooltip.textContent = audit.result.errorMessage || strings.errorMissingAuditInfo;
      } else if (audit.result.explanation) {
        var explEl = this.dom.createChildOf(titleEl, 'div', 'lh-audit-explanation');
        explEl.textContent = audit.result.explanation;
      }

      var warnings = audit.result.warnings;
      if (!warnings || warnings.length === 0) return auditEl; // Add list of warnings or singular warning

      var summaryEl = this.dom.find('summary', header);
      var warningsEl = this.dom.createChildOf(summaryEl, 'div', 'lh-warnings');
      this.dom.createChildOf(warningsEl, 'span').textContent = strings.warningHeader;

      if (warnings.length === 1) {
        warningsEl.appendChild(this.dom.document().createTextNode(warnings.join('')));
      } else {
        var warningsUl = this.dom.createChildOf(warningsEl, 'ul');

        var _iterator = _createForOfIteratorHelper(warnings),
            _step;

        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var warning = _step.value;
            var item = this.dom.createChildOf(warningsUl, 'li');
            item.textContent = warning;
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      }

      return auditEl;
    }
    /**
     * @return {HTMLElement}
     */

  }, {
    key: "_createChevron",
    value: function _createChevron() {
      var chevronTmpl = this.dom.cloneTemplate('#tmpl-lh-chevron', this.templateContext);
      var chevronEl = this.dom.find('.lh-chevron', chevronTmpl);
      return chevronEl;
    }
    /**
     * @param {Element} element DOM node to populate with values.
     * @param {number|null} score
     * @param {string} scoreDisplayMode
     * @return {!Element}
     */

  }, {
    key: "_setRatingClass",
    value: function _setRatingClass(element, score, scoreDisplayMode) {
      var rating = _util.default.calculateRating(score, scoreDisplayMode);

      element.classList.add("lh-audit--".concat(scoreDisplayMode.toLowerCase()));

      if (scoreDisplayMode !== 'informative') {
        element.classList.add("lh-audit--".concat(rating));
      }

      return element;
    }
    /**
     * @param {LH.ReportResult.Category} category
     * @param {Record<string, LH.Result.ReportGroup>} groupDefinitions
     * @return {Element}
     */

  }, {
    key: "renderCategoryHeader",
    value: function renderCategoryHeader(category, groupDefinitions) {
      var tmpl = this.dom.cloneTemplate('#tmpl-lh-category-header', this.templateContext);
      var gaugeContainerEl = this.dom.find('.lh-score__gauge', tmpl);
      var gaugeEl = this.renderScoreGauge(category, groupDefinitions);
      gaugeContainerEl.appendChild(gaugeEl);

      if (category.description) {
        var descEl = this.dom.convertMarkdownLinkSnippets(category.description);
        this.dom.find('.lh-category-header__description', tmpl).appendChild(descEl);
      }

      return (
        /** @type {Element} */
        tmpl.firstElementChild
      );
    }
    /**
     * Renders the group container for a group of audits. Individual audit elements can be added
     * directly to the returned element.
     * @param {LH.Result.ReportGroup} group
     * @return {Element}
     */

  }, {
    key: "renderAuditGroup",
    value: function renderAuditGroup(group) {
      var groupEl = this.dom.createElement('div', 'lh-audit-group');
      var auditGroupHeader = this.dom.createElement('div', 'lh-audit-group__header');
      this.dom.createChildOf(auditGroupHeader, 'span', 'lh-audit-group__title').textContent = group.title;

      if (group.description) {
        var descriptionEl = this.dom.convertMarkdownLinkSnippets(group.description);
        descriptionEl.classList.add('lh-audit-group__description');
        auditGroupHeader.appendChild(descriptionEl);
      }

      groupEl.appendChild(auditGroupHeader);
      return groupEl;
    }
    /**
     * Takes an array of auditRefs, groups them if requested, then returns an
     * array of audit and audit-group elements.
     * @param {Array<LH.ReportResult.AuditRef>} auditRefs
     * @param {Object<string, LH.Result.ReportGroup>} groupDefinitions
     * @return {Array<Element>}
     */

  }, {
    key: "_renderGroupedAudits",
    value: function _renderGroupedAudits(auditRefs, groupDefinitions) {
      // Audits grouped by their group (or under notAGroup).

      /** @type {Map<string, Array<LH.ReportResult.AuditRef>>} */
      var grouped = new Map(); // Add audits without a group first so they will appear first.

      var notAGroup = 'NotAGroup';
      grouped.set(notAGroup, []);

      var _iterator2 = _createForOfIteratorHelper(auditRefs),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var auditRef = _step2.value;
          var groupId = auditRef.group || notAGroup;
          var groupAuditRefs = grouped.get(groupId) || [];
          groupAuditRefs.push(auditRef);
          grouped.set(groupId, groupAuditRefs);
        }
        /** @type {Array<Element>} */

      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      var auditElements = [];

      var _iterator3 = _createForOfIteratorHelper(grouped),
          _step3;

      try {
        for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
          var _step3$value = _slicedToArray(_step3.value, 2),
              _groupId = _step3$value[0],
              _groupAuditRefs = _step3$value[1];

          if (_groupId === notAGroup) {
            // Push not-grouped audits individually.
            var _iterator4 = _createForOfIteratorHelper(_groupAuditRefs),
                _step4;

            try {
              for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
                var _auditRef = _step4.value;
                auditElements.push(this.renderAudit(_auditRef));
              }
            } catch (err) {
              _iterator4.e(err);
            } finally {
              _iterator4.f();
            }

            continue;
          } // Push grouped audits as a group.


          var groupDef = groupDefinitions[_groupId];
          var auditGroupElem = this.renderAuditGroup(groupDef);

          var _iterator5 = _createForOfIteratorHelper(_groupAuditRefs),
              _step5;

          try {
            for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
              var _auditRef2 = _step5.value;
              auditGroupElem.appendChild(this.renderAudit(_auditRef2));
            }
          } catch (err) {
            _iterator5.e(err);
          } finally {
            _iterator5.f();
          }

          auditGroupElem.classList.add("lh-audit-group--".concat(_groupId));
          auditElements.push(auditGroupElem);
        }
      } catch (err) {
        _iterator3.e(err);
      } finally {
        _iterator3.f();
      }

      return auditElements;
    }
    /**
     * Take a set of audits, group them if they have groups, then render in a top-level
     * clump that can't be expanded/collapsed.
     * @param {Array<LH.ReportResult.AuditRef>} auditRefs
     * @param {Object<string, LH.Result.ReportGroup>} groupDefinitions
     * @return {Element}
     */

  }, {
    key: "renderUnexpandableClump",
    value: function renderUnexpandableClump(auditRefs, groupDefinitions) {
      var clumpElement = this.dom.createElement('div');

      var elements = this._renderGroupedAudits(auditRefs, groupDefinitions);

      elements.forEach(function (elem) {
        return clumpElement.appendChild(elem);
      });
      return clumpElement;
    }
    /**
     * Take a set of audits and render in a top-level, expandable clump that starts
     * in a collapsed state.
     * @param {Exclude<TopLevelClumpId, 'failed'>} clumpId
     * @param {{auditRefs: Array<LH.ReportResult.AuditRef>, description?: string}} clumpOpts
     * @return {!Element}
     */

  }, {
    key: "renderClump",
    value: function renderClump(clumpId, _ref) {
      var auditRefs = _ref.auditRefs,
          description = _ref.description;
      var clumpTmpl = this.dom.cloneTemplate('#tmpl-lh-clump', this.templateContext);
      var clumpElement = this.dom.find('.lh-clump', clumpTmpl);

      if (clumpId === 'warning') {
        clumpElement.setAttribute('open', '');
      }

      var summaryInnerEl = this.dom.find('.lh-audit-group__summary', clumpElement);
      var chevronEl = summaryInnerEl.appendChild(this._createChevron());
      chevronEl.title = _util.default.i18n.strings.auditGroupExpandTooltip;
      var headerEl = this.dom.find('.lh-audit-group__header', clumpElement);
      var title = this._clumpTitles[clumpId];
      this.dom.find('.lh-audit-group__title', headerEl).textContent = title;

      if (description) {
        var descriptionEl = this.dom.convertMarkdownLinkSnippets(description);
        descriptionEl.classList.add('lh-audit-group__description');
        headerEl.appendChild(descriptionEl);
      }

      var itemCountEl = this.dom.find('.lh-audit-group__itemcount', clumpElement);
      itemCountEl.textContent = "(".concat(auditRefs.length, ")"); // Add all audit results to the clump.

      var auditElements = auditRefs.map(this.renderAudit.bind(this));
      clumpElement.append.apply(clumpElement, _toConsumableArray(auditElements));
      clumpElement.classList.add("lh-clump--".concat(clumpId.toLowerCase()));
      return clumpElement;
    }
    /**
     * @param {ParentNode} context
     */

  }, {
    key: "setTemplateContext",
    value: function setTemplateContext(context) {
      this.templateContext = context;
      this.detailsRenderer.setTemplateContext(context);
    }
    /**
     * @param {LH.ReportResult.Category} category
     * @param {Record<string, LH.Result.ReportGroup>} groupDefinitions
     * @return {DocumentFragment}
     */

  }, {
    key: "renderScoreGauge",
    value: function renderScoreGauge(category, groupDefinitions) {
      // eslint-disable-line no-unused-vars
      var tmpl = this.dom.cloneTemplate('#tmpl-lh-gauge', this.templateContext);
      var wrapper =
      /** @type {HTMLAnchorElement} */
      this.dom.find('.lh-gauge__wrapper', tmpl);
      wrapper.href = "#".concat(category.id);
      wrapper.classList.add("lh-gauge__wrapper--".concat(_util.default.calculateRating(category.score)));

      if (_util.default.isPluginCategory(category.id)) {
        wrapper.classList.add('lh-gauge__wrapper--plugin');
      } // Cast `null` to 0


      var numericScore = Number(category.score);
      var gauge = this.dom.find('.lh-gauge', tmpl);
      /** @type {?SVGCircleElement} */

      var gaugeArc = gauge.querySelector('.lh-gauge-arc');
      if (gaugeArc) this._setGaugeArc(gaugeArc, numericScore);
      var scoreOutOf100 = Math.round(numericScore * 100);
      var percentageEl = this.dom.find('.lh-gauge__percentage', tmpl);
      percentageEl.textContent = scoreOutOf100.toString();

      if (category.score === null) {
        percentageEl.textContent = '?';
        percentageEl.title = _util.default.i18n.strings.errorLabel;
      }

      this.dom.find('.lh-gauge__label', tmpl).textContent = category.title;
      return tmpl;
    }
    /**
     * Define the score arc of the gauge
     * Credit to xgad for the original technique: https://codepen.io/xgad/post/svg-radial-progress-meters
     * @param {SVGCircleElement} arcElem
     * @param {number} percent
     */

  }, {
    key: "_setGaugeArc",
    value: function _setGaugeArc(arcElem, percent) {
      var circumferencePx = 2 * Math.PI * Number(arcElem.getAttribute('r')); // The rounded linecap of the stroke extends the arc past its start and end.
      // First, we tweak the -90deg rotation to start exactly at the top of the circle.

      var strokeWidthPx = Number(arcElem.getAttribute('stroke-width'));
      var rotationalAdjustmentPercent = 0.25 * strokeWidthPx / circumferencePx;
      arcElem.style.transform = "rotate(".concat(-90 + rotationalAdjustmentPercent * 360, "deg)"); // Then, we terminate the line a little early as well.

      var arcLengthPx = percent * circumferencePx - strokeWidthPx / 2; // Special cases. No dot for 0, and full ring if 100

      if (percent === 0) arcElem.style.opacity = '0';
      if (percent === 1) arcLengthPx = circumferencePx;
      arcElem.style.strokeDasharray = "".concat(Math.max(arcLengthPx, 0), " ").concat(circumferencePx);
    }
    /**
     * @param {LH.ReportResult.AuditRef} audit
     * @return {boolean}
     */

  }, {
    key: "_auditHasWarning",
    value: function _auditHasWarning(audit) {
      return Boolean(audit.result.warnings && audit.result.warnings.length);
    }
    /**
     * Returns the id of the top-level clump to put this audit in.
     * @param {LH.ReportResult.AuditRef} auditRef
     * @return {TopLevelClumpId}
     */

  }, {
    key: "_getClumpIdForAuditRef",
    value: function _getClumpIdForAuditRef(auditRef) {
      var scoreDisplayMode = auditRef.result.scoreDisplayMode;

      if (scoreDisplayMode === 'manual' || scoreDisplayMode === 'notApplicable') {
        return scoreDisplayMode;
      }

      if (_util.default.showAsPassed(auditRef.result)) {
        if (this._auditHasWarning(auditRef)) {
          return 'warning';
        } else {
          return 'passed';
        }
      } else {
        return 'failed';
      }
    }
    /**
     * Renders a set of top level sections (clumps), under a status of failed, warning,
     * manual, passed, or notApplicable. The result ends up something like:
     *
     * failed clump
     *   ├── audit 1 (w/o group)
     *   ├── audit 2 (w/o group)
     *   ├── audit group
     *   |  ├── audit 3
     *   |  └── audit 4
     *   └── audit group
     *      ├── audit 5
     *      └── audit 6
     * other clump (e.g. 'manual')
     *   ├── audit 1
     *   ├── audit 2
     *   ├── …
     *   ⋮
     * @param {LH.ReportResult.Category} category
     * @param {Object<string, LH.Result.ReportGroup>} [groupDefinitions]
     * @return {Element}
     */

  }, {
    key: "render",
    value: function render(category) {
      var groupDefinitions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var element = this.dom.createElement('div', 'lh-category');
      this.createPermalinkSpan(element, category.id);
      element.appendChild(this.renderCategoryHeader(category, groupDefinitions)); // Top level clumps for audits, in order they will appear in the report.

      /** @type {Map<TopLevelClumpId, Array<LH.ReportResult.AuditRef>>} */

      var clumps = new Map();
      clumps.set('failed', []);
      clumps.set('warning', []);
      clumps.set('manual', []);
      clumps.set('passed', []);
      clumps.set('notApplicable', []); // Sort audits into clumps.

      var _iterator6 = _createForOfIteratorHelper(category.auditRefs),
          _step6;

      try {
        for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
          var auditRef = _step6.value;

          var clumpId = this._getClumpIdForAuditRef(auditRef);

          var clump =
          /** @type {Array<LH.ReportResult.AuditRef>} */
          clumps.get(clumpId); // already defined

          clump.push(auditRef);
          clumps.set(clumpId, clump);
        } // Render each clump.

      } catch (err) {
        _iterator6.e(err);
      } finally {
        _iterator6.f();
      }

      var _iterator7 = _createForOfIteratorHelper(clumps),
          _step7;

      try {
        for (_iterator7.s(); !(_step7 = _iterator7.n()).done;) {
          var _step7$value = _slicedToArray(_step7.value, 2),
              _clumpId = _step7$value[0],
              auditRefs = _step7$value[1];

          if (auditRefs.length === 0) continue;

          if (_clumpId === 'failed') {
            var _clumpElem = this.renderUnexpandableClump(auditRefs, groupDefinitions);

            _clumpElem.classList.add("lh-clump--failed");

            element.appendChild(_clumpElem);
            continue;
          }

          var description = _clumpId === 'manual' ? category.manualDescription : undefined;
          var clumpElem = this.renderClump(_clumpId, {
            auditRefs: auditRefs,
            description: description
          });
          element.appendChild(clumpElem);
        }
      } catch (err) {
        _iterator7.e(err);
      } finally {
        _iterator7.f();
      }

      return element;
    }
    /**
     * Create a non-semantic span used for hash navigation of categories
     * @param {Element} element
     * @param {string} id
     */

  }, {
    key: "createPermalinkSpan",
    value: function createPermalinkSpan(element, id) {
      var permalinkEl = this.dom.createChildOf(element, 'span', 'lh-permalink');
      permalinkEl.id = id;
    }
  }, {
    key: "_clumpTitles",
    get: function get() {
      return {
        warning: _util.default.i18n.strings.warningAuditsGroupTitle,
        manual: _util.default.i18n.strings.manualAuditsGroupTitle,
        passed: _util.default.i18n.strings.passedAuditsGroupTitle,
        notApplicable: _util.default.i18n.strings.notApplicableAuditsGroupTitle
      };
    }
  }]);

  return CategoryRenderer;
}();

var _default = CategoryRenderer;
exports.default = _default;