/**
 * @license
 * Copyright 2017 The Lighthouse Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _util = _interopRequireDefault(require("./util"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/* globals URL self Util */

/** @typedef {HTMLElementTagNameMap & {[id: string]: HTMLElement}} HTMLElementByTagName */
var DOM = /*#__PURE__*/function () {
  /**
   * @param {Document} document
   */
  function DOM(document) {
    _classCallCheck(this, DOM);

    /** @type {Document} */
    this._document = document;
    /** @type {string} */

    this._lighthouseChannel = 'unknown';
  }
  /**
   * @template {string} T
   * @param {T} name
   * @param {string=} className
   * @param {Object<string, (string|undefined)>=} attrs Attribute key/val pairs.
   *     Note: if an attribute key has an undefined value, this method does not
   *     set the attribute on the node.
   * @return {HTMLElementByTagName[T]}
   */


  _createClass(DOM, [{
    key: "createElement",
    value: function createElement(name, className) {
      var attrs = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      var element = this._document.createElement(name);

      if (className) {
        element.className = className;
      }

      Object.keys(attrs).forEach(function (key) {
        var value = attrs[key];

        if (typeof value !== 'undefined') {
          element.setAttribute(key, value);
        }
      });
      return element;
    }
    /**
     * @return {!DocumentFragment}
     */

  }, {
    key: "createFragment",
    value: function createFragment() {
      return this._document.createDocumentFragment();
    }
    /**
     * @template {string} T
     * @param {Element} parentElem
     * @param {T} elementName
     * @param {string=} className
     * @param {Object<string, (string|undefined)>=} attrs Attribute key/val pairs.
     *     Note: if an attribute key has an undefined value, this method does not
     *     set the attribute on the node.
     * @return {HTMLElementByTagName[T]}
     */

  }, {
    key: "createChildOf",
    value: function createChildOf(parentElem, elementName, className, attrs) {
      var element = this.createElement(elementName, className, attrs);
      parentElem.appendChild(element);
      return element;
    }
    /**
     * @param {string} selector
     * @param {ParentNode} context
     * @return {!DocumentFragment} A clone of the template content.
     * @throws {Error}
     */

  }, {
    key: "cloneTemplate",
    value: function cloneTemplate(selector, context) {
      var template =
      /** @type {?HTMLTemplateElement} */
      context.querySelector(selector);

      if (!template) {
        throw new Error("Template not found: template".concat(selector));
      }

      var clone = this._document.importNode(template.content, true); // Prevent duplicate styles in the DOM. After a template has been stamped
      // for the first time, remove the clone's styles so they're not re-added.


      if (template.hasAttribute('data-stamped')) {
        this.findAll('style', clone).forEach(function (style) {
          return style.remove();
        });
      }

      template.setAttribute('data-stamped', 'true');
      return clone;
    }
    /**
     * Resets the "stamped" state of the templates.
     */

  }, {
    key: "resetTemplates",
    value: function resetTemplates() {
      this.findAll('template[data-stamped]', this._document).forEach(function (t) {
        t.removeAttribute('data-stamped');
      });
    }
    /**
     * @param {string} text
     * @return {Element}
     */

  }, {
    key: "convertMarkdownLinkSnippets",
    value: function convertMarkdownLinkSnippets(text) {
      var element = this.createElement('span');

      var _iterator = _createForOfIteratorHelper(_util.default.splitMarkdownLink(text)),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var segment = _step.value;

          if (!segment.isLink) {
            // Plain text segment.
            element.appendChild(this._document.createTextNode(segment.text));
            continue;
          } // Otherwise, append any links found.


          var url = new URL(segment.linkHref);
          var DOCS_ORIGINS = ['https://developers.google.com', 'https://web.dev'];

          if (DOCS_ORIGINS.includes(url.origin)) {
            url.searchParams.set('utm_source', 'lighthouse');
            url.searchParams.set('utm_medium', this._lighthouseChannel);
          }

          var a = this.createElement('a');
          a.rel = 'noopener';
          a.target = '_blank';
          a.textContent = segment.text;
          a.href = url.href;
          element.appendChild(a);
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return element;
    }
    /**
     * @param {string} markdownText
     * @return {Element}
     */

  }, {
    key: "convertMarkdownCodeSnippets",
    value: function convertMarkdownCodeSnippets(markdownText) {
      var element = this.createElement('span');

      var _iterator2 = _createForOfIteratorHelper(_util.default.splitMarkdownCodeSpans(markdownText)),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var segment = _step2.value;

          if (segment.isCode) {
            var pre = this.createElement('code');
            pre.textContent = segment.text;
            element.appendChild(pre);
          } else {
            element.appendChild(this._document.createTextNode(segment.text));
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      return element;
    }
    /**
     * The channel to use for UTM data when rendering links to the documentation.
     * @param {string} lighthouseChannel
     */

  }, {
    key: "setLighthouseChannel",
    value: function setLighthouseChannel(lighthouseChannel) {
      this._lighthouseChannel = lighthouseChannel;
    }
    /**
     * @return {Document}
     */

  }, {
    key: "document",
    value: function document() {
      return this._document;
    }
    /**
     * TODO(paulirish): import and conditionally apply the DevTools frontend subclasses instead of this
     * @return {boolean}
     */

  }, {
    key: "isDevTools",
    value: function isDevTools() {
      return !!this._document.querySelector('.lh-devtools');
    }
    /**
     * Guaranteed context.querySelector. Always returns an element or throws if
     * nothing matches query.
     * @param {string} query
     * @param {ParentNode} context
     * @return {!HTMLElement}
     */

  }, {
    key: "find",
    value: function find(query, context) {
      /** @type {?HTMLElement} */
      var result = context.querySelector(query);

      if (result === null) {
        throw new Error("query ".concat(query, " not found"));
      }

      return result;
    }
    /**
     * Helper for context.querySelectorAll. Returns an Array instead of a NodeList.
     * @param {string} query
     * @param {ParentNode} context
     * @return {!Array<HTMLElement>}
     */

  }, {
    key: "findAll",
    value: function findAll(query, context) {
      return Array.from(context.querySelectorAll(query));
    }
  }]);

  return DOM;
}();

var _default = DOM;
exports.default = _default;