/**
 * @license
 * Copyright 2018 The Lighthouse Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _categoryRenderer = _interopRequireDefault(require("./category-renderer.js"));

var _util = _interopRequireDefault(require("./util.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/* globals self, Util, CategoryRenderer */

/** @typedef {import('./dom.js')} DOM */
var PerformanceCategoryRenderer = /*#__PURE__*/function (_CategoryRenderer) {
  _inherits(PerformanceCategoryRenderer, _CategoryRenderer);

  var _super = _createSuper(PerformanceCategoryRenderer);

  function PerformanceCategoryRenderer() {
    _classCallCheck(this, PerformanceCategoryRenderer);

    return _super.apply(this, arguments);
  }

  _createClass(PerformanceCategoryRenderer, [{
    key: "_renderMetric",

    /**
     * @param {LH.ReportResult.AuditRef} audit
     * @return {!Element}
     */
    value: function _renderMetric(audit) {
      var tmpl = this.dom.cloneTemplate('#tmpl-lh-metric', this.templateContext);
      var element = this.dom.find('.lh-metric', tmpl);
      element.id = audit.result.id;

      var rating = _util.default.calculateRating(audit.result.score, audit.result.scoreDisplayMode);

      element.classList.add("lh-metric--".concat(rating));
      var titleEl = this.dom.find('.lh-metric__title', tmpl);
      titleEl.textContent = audit.result.title;
      var valueEl = this.dom.find('.lh-metric__value', tmpl);
      valueEl.textContent = audit.result.displayValue || '';
      var descriptionEl = this.dom.find('.lh-metric__description', tmpl);
      descriptionEl.appendChild(this.dom.convertMarkdownLinkSnippets(audit.result.description));

      if (audit.result.scoreDisplayMode === 'error') {
        descriptionEl.textContent = '';
        valueEl.textContent = 'Error!';
        var tooltip = this.dom.createChildOf(descriptionEl, 'span');
        tooltip.textContent = audit.result.errorMessage || 'Report error: no metric information';
      }

      return element;
    }
    /**
     * @param {LH.ReportResult.AuditRef} audit
     * @param {number} scale
     * @return {!Element}
     */

  }, {
    key: "_renderOpportunity",
    value: function _renderOpportunity(audit, scale) {
      var oppTmpl = this.dom.cloneTemplate('#tmpl-lh-opportunity', this.templateContext);
      var element = this.populateAuditValues(audit, oppTmpl);
      element.id = audit.result.id;

      if (!audit.result.details || audit.result.scoreDisplayMode === 'error') {
        return element;
      }

      var details = audit.result.details;

      if (details.type !== 'opportunity') {
        return element;
      } // Overwrite the displayValue with opportunity's wastedMs


      var displayEl = this.dom.find('.lh-audit__display-text', element);
      var sparklineWidthPct = "".concat(details.overallSavingsMs / scale * 100, "%");
      this.dom.find('.lh-sparkline__bar', element).style.width = sparklineWidthPct;
      displayEl.textContent = _util.default.i18n.formatSeconds(details.overallSavingsMs, 0.01); // Set [title] tooltips

      if (audit.result.displayValue) {
        var displayValue = audit.result.displayValue;
        this.dom.find('.lh-load-opportunity__sparkline', element).title = displayValue;
        displayEl.title = displayValue;
      }

      return element;
    }
    /**
     * Get an audit's wastedMs to sort the opportunity by, and scale the sparkline width
     * Opportunities with an error won't have a details object, so MIN_VALUE is returned to keep any
     * erroring opportunities last in sort order.
     * @param {LH.ReportResult.AuditRef} audit
     * @return {number}
     */

  }, {
    key: "_getWastedMs",
    value: function _getWastedMs(audit) {
      if (audit.result.details && audit.result.details.type === 'opportunity') {
        var details = audit.result.details;

        if (typeof details.overallSavingsMs !== 'number') {
          throw new Error('non-opportunity details passed to _getWastedMs');
        }

        return details.overallSavingsMs;
      } else {
        return Number.MIN_VALUE;
      }
    }
    /**
     * Get a link to the interactive scoring calculator with the metric values.
     * @param {LH.ReportResult.AuditRef[]} auditRefs
     * @return {string}
     */

  }, {
    key: "_getScoringCalculatorHref",
    value: function _getScoringCalculatorHref(auditRefs) {
      var v5andv6metrics = auditRefs.filter(function (audit) {
        return audit.group === 'metrics';
      });
      var fci = auditRefs.find(function (audit) {
        return audit.id === 'first-cpu-idle';
      });
      var fmp = auditRefs.find(function (audit) {
        return audit.id === 'first-meaningful-paint';
      });
      if (fci) v5andv6metrics.push(fci);
      if (fmp) v5andv6metrics.push(fmp);
      /** @type {Record<string, string>} */

      var acronymMapping = {
        'cumulative-layout-shift': 'CLS',
        'first-contentful-paint': 'FCP',
        'first-cpu-idle': 'FCI',
        'first-meaningful-paint': 'FMP',
        interactive: 'TTI',
        'largest-contentful-paint': 'LCP',
        'speed-index': 'SI',
        'total-blocking-time': 'TBT'
      };
      /**
       * Clamp figure to 2 decimal places
       * @param {number} val
       * @return {number}
       */

      var clampTo2Decimals = function clampTo2Decimals(val) {
        return Math.round(val * 100) / 100;
      };

      var metricPairs = v5andv6metrics.map(function (audit) {
        var value;

        if (typeof audit.result.numericValue === 'number') {
          value = audit.id === 'cumulative-layout-shift' ? clampTo2Decimals(audit.result.numericValue) : Math.round(audit.result.numericValue);
          value = value.toString();
        } else {
          value = 'null';
        }

        return [acronymMapping[audit.id] || audit.id, value];
      });

      var paramPairs = _toConsumableArray(metricPairs);

      if (_util.default.reportJson) {
        paramPairs.push(['device', _util.default.reportJson.configSettings.emulatedFormFactor]);
        paramPairs.push(['version', _util.default.reportJson.lighthouseVersion]);
      }

      var params = new URLSearchParams(paramPairs);
      var url = new URL('https://googlechrome.github.io/lighthouse/scorecalc/');
      url.hash = params.toString();
      return url.href;
    }
    /**
     * @param {LH.ReportResult.Category} category
     * @param {Object<string, LH.Result.ReportGroup>} groups
     * @param {'PSI'=} environment 'PSI' and undefined are the only valid values
     * @return {Element}
     * @override
     */

  }, {
    key: "render",
    value: function render(category, groups, environment) {
      var _this = this;

      var strings = _util.default.i18n.strings;
      var element = this.dom.createElement('div', 'lh-category');

      if (environment === 'PSI') {
        var gaugeEl = this.dom.createElement('div', 'lh-score__gauge');
        gaugeEl.appendChild(this.renderScoreGauge(category, groups));
        element.appendChild(gaugeEl);
      } else {
        this.createPermalinkSpan(element, category.id);
        element.appendChild(this.renderCategoryHeader(category, groups));
      } // Metrics.


      var metricAuditsEl = this.renderAuditGroup(groups.metrics); // Metric descriptions toggle.

      var toggleTmpl = this.dom.cloneTemplate('#tmpl-lh-metrics-toggle', this.templateContext);

      var _toggleEl = this.dom.find('.lh-metrics-toggle', toggleTmpl);

      metricAuditsEl.append.apply(metricAuditsEl, _toConsumableArray(_toggleEl.childNodes));
      var metricAudits = category.auditRefs.filter(function (audit) {
        return audit.group === 'metrics';
      });
      var metricsBoxesEl = this.dom.createChildOf(metricAuditsEl, 'div', 'lh-metrics-container');
      metricAudits.forEach(function (item) {
        metricsBoxesEl.appendChild(_this._renderMetric(item));
      });
      var estValuesEl = this.dom.createChildOf(metricAuditsEl, 'div', 'lh-metrics__disclaimer');
      var disclaimerEl = this.dom.convertMarkdownLinkSnippets(strings.varianceDisclaimer);
      estValuesEl.appendChild(disclaimerEl); // Add link to score calculator.

      var calculatorLink = this.dom.createChildOf(estValuesEl, 'a', 'lh-calclink');
      calculatorLink.target = '_blank';
      calculatorLink.textContent = strings.calculatorLink;
      calculatorLink.href = this._getScoringCalculatorHref(category.auditRefs);
      metricAuditsEl.classList.add('lh-audit-group--metrics');
      element.appendChild(metricAuditsEl); // Filmstrip

      var timelineEl = this.dom.createChildOf(element, 'div', 'lh-filmstrip-container');
      var thumbnailAudit = category.auditRefs.find(function (audit) {
        return audit.id === 'screenshot-thumbnails';
      });
      var thumbnailResult = thumbnailAudit && thumbnailAudit.result;

      if (thumbnailResult && thumbnailResult.details) {
        timelineEl.id = thumbnailResult.id;
        var filmstripEl = this.detailsRenderer.render(thumbnailResult.details);
        filmstripEl && timelineEl.appendChild(filmstripEl);
      } // Budgets

      /** @type {Array<Element>} */


      var budgetTableEls = [];
      ['performance-budget', 'timing-budget'].forEach(function (id) {
        var audit = category.auditRefs.find(function (audit) {
          return audit.id === id;
        });

        if (audit && audit.result.details) {
          var table = _this.detailsRenderer.render(audit.result.details);

          if (table) {
            table.id = id;
            table.classList.add('lh-audit');
            budgetTableEls.push(table);
          }
        }
      });

      if (budgetTableEls.length > 0) {
        var budgetsGroupEl = this.renderAuditGroup(groups.budgets);
        budgetTableEls.forEach(function (table) {
          return budgetsGroupEl.appendChild(table);
        });
        budgetsGroupEl.classList.add('lh-audit-group--budgets');
        element.appendChild(budgetsGroupEl);
      } // Opportunities


      var opportunityAudits = category.auditRefs.filter(function (audit) {
        return audit.group === 'load-opportunities' && !_util.default.showAsPassed(audit.result);
      }).sort(function (auditA, auditB) {
        return _this._getWastedMs(auditB) - _this._getWastedMs(auditA);
      });

      if (opportunityAudits.length) {
        // Scale the sparklines relative to savings, minimum 2s to not overstate small savings
        var minimumScale = 2000;
        var wastedMsValues = opportunityAudits.map(function (audit) {
          return _this._getWastedMs(audit);
        });
        var maxWaste = Math.max.apply(Math, _toConsumableArray(wastedMsValues));
        var scale = Math.max(Math.ceil(maxWaste / 1000) * 1000, minimumScale);
        var groupEl = this.renderAuditGroup(groups['load-opportunities']);
        var tmpl = this.dom.cloneTemplate('#tmpl-lh-opportunity-header', this.templateContext);
        this.dom.find('.lh-load-opportunity__col--one', tmpl).textContent = strings.opportunityResourceColumnLabel;
        this.dom.find('.lh-load-opportunity__col--two', tmpl).textContent = strings.opportunitySavingsColumnLabel;
        var headerEl = this.dom.find('.lh-load-opportunity__header', tmpl);
        groupEl.appendChild(headerEl);
        opportunityAudits.forEach(function (item) {
          return groupEl.appendChild(_this._renderOpportunity(item, scale));
        });
        groupEl.classList.add('lh-audit-group--load-opportunities');
        element.appendChild(groupEl);
      } // Diagnostics


      var diagnosticAudits = category.auditRefs.filter(function (audit) {
        return audit.group === 'diagnostics' && !_util.default.showAsPassed(audit.result);
      }).sort(function (a, b) {
        var scoreA = a.result.scoreDisplayMode === 'informative' ? 100 : Number(a.result.score);
        var scoreB = b.result.scoreDisplayMode === 'informative' ? 100 : Number(b.result.score);
        return scoreA - scoreB;
      });

      if (diagnosticAudits.length) {
        var _groupEl = this.renderAuditGroup(groups['diagnostics']);

        diagnosticAudits.forEach(function (item) {
          return _groupEl.appendChild(_this.renderAudit(item));
        });

        _groupEl.classList.add('lh-audit-group--diagnostics');

        element.appendChild(_groupEl);
      } // Passed audits


      var passedAudits = category.auditRefs.filter(function (audit) {
        return (audit.group === 'load-opportunities' || audit.group === 'diagnostics') && _util.default.showAsPassed(audit.result);
      });
      if (!passedAudits.length) return element;
      var clumpOpts = {
        auditRefs: passedAudits,
        groupDefinitions: groups
      };
      var passedElem = this.renderClump('passed', clumpOpts);
      element.appendChild(passedElem);
      return element;
    }
  }]);

  return PerformanceCategoryRenderer;
}(_categoryRenderer.default);

var _default = PerformanceCategoryRenderer;
exports.default = _default;