/**
 * @license
 * Copyright 2017 The Lighthouse Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _util = _interopRequireDefault(require("./util"));

var _i18n = _interopRequireDefault(require("./i18n"));

var _detailsRenderer = _interopRequireDefault(require("./details-renderer"));

var _categoryRenderer = _interopRequireDefault(require("./category-renderer"));

var _performanceCategoryRenderer = _interopRequireDefault(require("./performance-category-renderer"));

var _pwaCategoryRenderer = _interopRequireDefault(require("./pwa-category-renderer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * @fileoverview The entry point for rendering the Lighthouse report based on the JSON output.
 *    This file is injected into the report HTML along with the JSON report.
 *
 * Dummy text for ensuring report robustness: </script> pre$`post %%LIGHTHOUSE_JSON%%
 */

/** @typedef {import('./category-renderer')} CategoryRenderer */

/** @typedef {import('./dom.js')} DOM */

/* globals self, Util, DetailsRenderer, CategoryRenderer, I18n, PerformanceCategoryRenderer, PwaCategoryRenderer */
var ReportRenderer = /*#__PURE__*/function () {
  /**
   * @param {DOM} dom
   */
  function ReportRenderer(dom) {
    _classCallCheck(this, ReportRenderer);

    /** @type {DOM} */
    this._dom = dom;
    /** @type {ParentNode} */

    this._templateContext = this._dom.document();
  }
  /**
   * @param {LH.Result} result
   * @param {Element} container Parent element to render the report into.
   * @return {!Element}
   */


  _createClass(ReportRenderer, [{
    key: "renderReport",
    value: function renderReport(result, container) {
      this._dom.setLighthouseChannel(result.configSettings.channel || 'unknown');

      var report = _util.default.prepareReportResult(result);

      container.textContent = ''; // Remove previous report.

      container.appendChild(this._renderReport(report));
      return container;
    }
    /**
     * Define a custom element for <templates> to be extracted from. For example:
     *     this.setTemplateContext(new DOMParser().parseFromString(htmlStr, 'text/html'))
     * @param {ParentNode} context
     */

  }, {
    key: "setTemplateContext",
    value: function setTemplateContext(context) {
      this._templateContext = context;
    }
    /**
     * @param {LH.ReportResult} report
     * @return {DocumentFragment}
     */

  }, {
    key: "_renderReportTopbar",
    value: function _renderReportTopbar(report) {
      var el = this._dom.cloneTemplate('#tmpl-lh-topbar', this._templateContext);

      var metadataUrl =
      /** @type {HTMLAnchorElement} */
      this._dom.find('.lh-topbar__url', el);

      metadataUrl.href = metadataUrl.textContent = report.finalUrl;
      metadataUrl.title = report.finalUrl;
      return el;
    }
    /**
     * @return {DocumentFragment}
     */

  }, {
    key: "_renderReportHeader",
    value: function _renderReportHeader() {
      var el = this._dom.cloneTemplate('#tmpl-lh-heading', this._templateContext);

      var domFragment = this._dom.cloneTemplate('#tmpl-lh-scores-wrapper', this._templateContext);

      var placeholder = this._dom.find('.lh-scores-wrapper-placeholder', el);
      /** @type {HTMLDivElement} */


      placeholder.parentNode.replaceChild(domFragment, placeholder);
      return el;
    }
    /**
     * @param {LH.ReportResult} report
     * @return {DocumentFragment}
     */

  }, {
    key: "_renderReportFooter",
    value: function _renderReportFooter(report) {
      var _this = this;

      var footer = this._dom.cloneTemplate('#tmpl-lh-footer', this._templateContext);

      var env = this._dom.find('.lh-env__items', footer);

      env.id = 'runtime-settings';
      this._dom.find('.lh-env__title', footer).textContent = _util.default.i18n.strings.runtimeSettingsTitle;

      var envValues = _util.default.getEnvironmentDisplayValues(report.configSettings || {});

      [{
        name: _util.default.i18n.strings.runtimeSettingsUrl,
        description: report.finalUrl
      }, {
        name: _util.default.i18n.strings.runtimeSettingsFetchTime,
        description: _util.default.i18n.formatDateTime(report.fetchTime)
      }].concat(_toConsumableArray(envValues), [{
        name: _util.default.i18n.strings.runtimeSettingsChannel,
        description: report.configSettings.channel
      }, {
        name: _util.default.i18n.strings.runtimeSettingsUA,
        description: report.userAgent
      }, {
        name: _util.default.i18n.strings.runtimeSettingsUANetwork,
        description: report.environment && report.environment.networkUserAgent
      }, {
        name: _util.default.i18n.strings.runtimeSettingsBenchmark,
        description: report.environment && report.environment.benchmarkIndex.toFixed(0)
      }]).forEach(function (runtime) {
        if (!runtime.description) return;

        var item = _this._dom.cloneTemplate('#tmpl-lh-env__items', env);

        _this._dom.find('.lh-env__name', item).textContent = runtime.name;
        _this._dom.find('.lh-env__description', item).textContent = runtime.description;
        env.appendChild(item);
      });
      this._dom.find('.lh-footer__version_issue', footer).textContent = _util.default.i18n.strings.footerIssue;
      this._dom.find('.lh-footer__version', footer).textContent = report.lighthouseVersion;
      return footer;
    }
    /**
     * Returns a div with a list of top-level warnings, or an empty div if no warnings.
     * @param {LH.ReportResult} report
     * @return {Node}
     */

  }, {
    key: "_renderReportWarnings",
    value: function _renderReportWarnings(report) {
      if (!report.runWarnings || report.runWarnings.length === 0) {
        return this._dom.createElement('div');
      }

      var container = this._dom.cloneTemplate('#tmpl-lh-warnings--toplevel', this._templateContext);

      var message = this._dom.find('.lh-warnings__msg', container);

      message.textContent = _util.default.i18n.strings.toplevelWarningsMessage;

      var warnings = this._dom.find('ul', container);

      var _iterator = _createForOfIteratorHelper(report.runWarnings),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var warningString = _step.value;
          var warning = warnings.appendChild(this._dom.createElement('li'));
          warning.textContent = warningString;
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return container;
    }
    /**
     * @param {LH.ReportResult} report
     * @param {CategoryRenderer} categoryRenderer
     * @param {Record<string, CategoryRenderer>} specificCategoryRenderers
     * @return {!DocumentFragment[]}
     */

  }, {
    key: "_renderScoreGauges",
    value: function _renderScoreGauges(report, categoryRenderer, specificCategoryRenderers) {
      // Group gauges in this order: default, pwa, plugins.
      var defaultGauges = [];
      var customGauges = []; // PWA.

      var pluginGauges = [];

      for (var _i = 0, _Object$values = Object.values(report.categories); _i < _Object$values.length; _i++) {
        var category = _Object$values[_i];
        var renderer = specificCategoryRenderers[category.id] || categoryRenderer;
        var categoryGauge = renderer.renderScoreGauge(category, report.categoryGroups || {});

        if (_util.default.isPluginCategory(category.id)) {
          pluginGauges.push(categoryGauge);
        } else if (renderer.renderScoreGauge === categoryRenderer.renderScoreGauge) {
          // The renderer for default categories is just the default CategoryRenderer.
          // If the functions are equal, then renderer is an instance of CategoryRenderer.
          // For example, the PWA category uses PwaCategoryRenderer, which overrides
          // CategoryRenderer.renderScoreGauge, so it would fail this check and be placed
          // in the customGauges bucket.
          defaultGauges.push(categoryGauge);
        } else {
          customGauges.push(categoryGauge);
        }
      }

      return [].concat(defaultGauges, customGauges, pluginGauges);
    }
    /**
     * @param {LH.ReportResult} report
     * @return {!DocumentFragment}
     */

  }, {
    key: "_renderReport",
    value: function _renderReport(report) {
      var _this2 = this;

      var i18n = new _i18n.default(report.configSettings.locale, _objectSpread(_objectSpread({}, _util.default.UIStrings), report.i18n.rendererFormattedStrings));
      _util.default.i18n = i18n;
      _util.default.reportJson = report;
      var detailsRenderer = new _detailsRenderer.default(this._dom);
      var categoryRenderer = new _categoryRenderer.default(this._dom, detailsRenderer);
      categoryRenderer.setTemplateContext(this._templateContext);
      /** @type {Record<string, CategoryRenderer>} */

      var specificCategoryRenderers = {
        performance: new _performanceCategoryRenderer.default(this._dom, detailsRenderer),
        pwa: new _pwaCategoryRenderer.default(this._dom, detailsRenderer)
      };
      Object.values(specificCategoryRenderers).forEach(function (renderer) {
        renderer.setTemplateContext(_this2._templateContext);
      });

      var headerContainer = this._dom.createElement('div');

      headerContainer.appendChild(this._renderReportHeader());

      var reportContainer = this._dom.createElement('div', 'lh-container');

      var reportSection = this._dom.createElement('div', 'lh-report');

      reportSection.appendChild(this._renderReportWarnings(report));
      var scoreHeader;
      var isSoloCategory = Object.keys(report.categories).length === 1;

      if (!isSoloCategory) {
        scoreHeader = this._dom.createElement('div', 'lh-scores-header');
      } else {
        headerContainer.classList.add('lh-header--solo-category');
      }

      if (scoreHeader) {
        var _scoreHeader;

        var scoreScale = this._dom.cloneTemplate('#tmpl-lh-scorescale', this._templateContext);

        var scoresContainer = this._dom.find('.lh-scores-container', headerContainer);

        (_scoreHeader = scoreHeader).append.apply(_scoreHeader, _toConsumableArray(this._renderScoreGauges(report, categoryRenderer, specificCategoryRenderers)));

        scoresContainer.appendChild(scoreHeader);
        scoresContainer.appendChild(scoreScale);

        var stickyHeader = this._dom.createElement('div', 'lh-sticky-header');

        stickyHeader.append.apply(stickyHeader, _toConsumableArray(this._renderScoreGauges(report, categoryRenderer, specificCategoryRenderers)));
        reportContainer.appendChild(stickyHeader);
      }

      var categories = reportSection.appendChild(this._dom.createElement('div', 'lh-categories'));

      for (var _i2 = 0, _Object$values2 = Object.values(report.categories); _i2 < _Object$values2.length; _i2++) {
        var category = _Object$values2[_i2];
        var renderer = specificCategoryRenderers[category.id] || categoryRenderer; // .lh-category-wrapper is full-width and provides horizontal rules between categories.
        // .lh-category within has the max-width: var(--report-width);

        var wrapper = renderer.dom.createChildOf(categories, 'div', 'lh-category-wrapper');
        wrapper.appendChild(renderer.render(category, report.categoryGroups));
      }

      var reportFragment = this._dom.createFragment();

      var topbarDocumentFragment = this._renderReportTopbar(report);

      reportFragment.appendChild(topbarDocumentFragment);
      reportFragment.appendChild(reportContainer);
      reportContainer.appendChild(headerContainer);
      reportContainer.appendChild(reportSection);
      reportSection.appendChild(this._renderReportFooter(report));
      return reportFragment;
    }
  }]);

  return ReportRenderer;
}();

var _default = ReportRenderer;
exports.default = _default;